package cn.iocoder.yudao.module.delivery.service.Impl;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.module.delivery.entity.CustomerAnalysisReq;
import cn.iocoder.yudao.module.delivery.entity.CustomerAnalysisResp;
import cn.iocoder.yudao.module.delivery.entity.CustomerAnalysisExcelResp;
import cn.iocoder.yudao.module.delivery.entity.CustomerReportReq;
import cn.iocoder.yudao.module.delivery.mapper.CustomerAnalysisMapper;
import cn.iocoder.yudao.module.delivery.service.CustomerAnalysisService;
import cn.iocoder.yudao.module.delivery.service.CustomerReportService;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.*;
import java.util.stream.Collectors;

/*
 lanbm 2024-04-01 add
 客户分析列表
 */
@Service("CustomerAnalysisService")
public class CustomerAnalysisImpl implements CustomerAnalysisService {

    @Autowired
    private CustomerAnalysisMapper customerAnalysisMapper;

    @Autowired
    private DictDataApi dictDataApi;

    @Resource
    private RedisHelper redisHelper;

    public List<CustomerAnalysisResp> getList(CustomerAnalysisReq req) {
        return customerAnalysisMapper.getList(req);
    }


    public PageResult<CustomerAnalysisResp> getListPage(CustomerAnalysisReq req)
            throws JsonProcessingException {

        IPage<CustomerAnalysisResp> mpPage = MyBatisUtils.buildPage(req);
        long total = customerAnalysisMapper.GetCount(req);
        System.out.println("符合条件的客户总数：" + String.valueOf(total));
        int start = (req.getPageNo() - 1) * req.getPageSize();
        int size = req.getPageSize();

        req.setSize(size);
        req.setStart(start);

        /*
        List<CustomerAnalysisResp> list = null;
        String sList = redisHelper.get("CustomerAnalysisRespList");
        if (sList == null || sList.length() == 0) {
            list = customerAnalysisMapper.getListPage(req);
            ObjectMapper objectMapper = new ObjectMapper();
            // 将List转换为JSON字符串
            String json = objectMapper.writeValueAsString(list);
            redisHelper.set("CustomerAnalysisRespList", json);
        } else {
            list = JSON.parseArray(sList, CustomerAnalysisResp.class);
        }
        */
        List<CustomerAnalysisResp> list = customerAnalysisMapper.getListPage(req);
        System.out.println("符合条件的客户总数：" + String.valueOf(total));
        return new PageResult<>(list, total, mpPage.getSize(),
                req.getPageNo(), (total + mpPage.getSize() - 1) / mpPage.getSize());

    }


    private List<CustomerAnalysisResp> GetPageResult(
            List<CustomerAnalysisResp> list, int pageNum, int pageSize) {
        /*
        // 正序
        Collections.sort(list, (a, b) -> {
            return a.排序字段 - b.排序字段;
        });
        // 倒序
        Collections.sort(list, (a, b) -> {
            return b.排序字段 - a.排序字段;
        });*/
        // 总数
        int total = list.size();

        // 分页数据
        List<CustomerAnalysisResp> subList =
                list.stream().skip((pageNum - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
        return subList;

    }


    /*
     计算百分比
     lanbm 2024-04-10 add
     */
    private String getPercentage(BigDecimal part, BigDecimal total) {
        BigDecimal percentage = part.divide(total, 5,
                BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("100"));
        // 输出百分比
        String sR = format(percentage) + "%";
        return sR;
    }

    /*
     格式化BigDecimal为字符串，保留两位小数
     lanbm 2024-04-10 add
     */
    private String format(BigDecimal value) {
        NumberFormat numberFormat = NumberFormat.getInstance();
        numberFormat.setMinimumFractionDigits(2);
        numberFormat.setMaximumFractionDigits(2);
        return numberFormat.format(value.doubleValue());
    }

    private String getCurDate() {
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String currentDate = sdf.format(date);
        return currentDate;
    }

    private int getCurYear() {
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        return year;
    }

    /*
     获取数据查询条件
     */
    public CustomerAnalysisReq getDataWhere(CustomerAnalysisReq query) {
        if (query.getSearchDataType1() != null &&
                query.getSearchDataType2() != null &&
                query.getSearchDataType3() != null) {

        }
        return query;
    }

    /*
     对查询逻辑做处理 lanbm 2024-04-16 add
     */
    public CustomerAnalysisReq getReq(CustomerAnalysisReq Req) {
        //计算月份差
        long lc = monthDiff(Req.getSdate(), Req.getEdate());
        if (lc == 0) {
            lc = 1;
        }
        Req.setNMonth(Integer.parseInt(String.valueOf(lc)));

        int y = getCurYear();
        if (Req.getSdate() == null) {
            Req.setCurYear(String.valueOf(y));
            Req.setSdate(String.valueOf(y) + "-01-01");

            if (Req.getDuibiYear() == null) {
                Req.setDuibiYear(String.valueOf(y - 1));
            } else {
                Req.setSduibidate(Req.getDuibiYear() + "-01-01");
            }

        } else {
            //2024-03-01
            String sT = Req.getSdate();
            String s = sT.substring(4, 10);

            if (Req.getDuibiYear() == null) {
                int nTemp = Integer.parseInt(sT.substring(0, 4)) - 1;
                Req.setSduibidate(String.valueOf(nTemp) + s);
            } else {
                Req.setSduibidate(Req.getDuibiYear() + s);
            }
            Req.setCurYear(sT.substring(0, 4));
        }

        if (Req.getEdate() == null) {
            Req.setEdate(getCurDate());

            String s = Req.getEdate().substring(5, 7);
            int ld = getLastDay(Integer.parseInt(Req.getDuibiYear()), Integer.parseInt(s));
            Req.setEduibidate(Req.getDuibiYear() + "-" + s + "-" + String.valueOf(ld));

        } else {
            String sE = Req.getEdate();
            String sY = sE.substring(0, 4);

            //int ld = getLastDay(Integer.parseInt(sY), Integer.parseInt(s));
            //2024-03-01
            String s1 = sE.substring(4, 10);
            if (Req.getDuibiYear() == null) {
                int nTemp = Integer.parseInt(sY) - 1;
                Req.setEduibidate(String.valueOf(nTemp) + s1);
            } else {
                Req.setEduibidate(Req.getDuibiYear() + s1);
            }
        }

        /*
        if (Req.getSearchData3() != null) {
            Req.setSearchData3(Req.getSearchData3().divide(new BigDecimal(100)));
        }*/
        return Req;
    }


    /*
    获取某年某月的最后一天 lanbm 2024-04-16 add
    */
    private int getLastDay(int year, int month) {
        YearMonth yearMonth = YearMonth.of(year, month);
        int lastDay = yearMonth.lengthOfMonth();
        return lastDay;
    }


    public PageResult<CustomerAnalysisResp> GetDataResult(CustomerAnalysisReq query)
            throws JsonProcessingException {

        query = getReq(query);
        query = getDataWhere(query);
        System.out.println("处理后的查询条件：" + query);

        //把model转 JSON字符串 lanbm
        //String jsonStr = JsonUtils.toJsonString(query);
        //JsonUtils.SaveLog(jsonStr);

        List<DictDataRespDTO> dicCustomer_type =
                dictDataApi.getDictDatas("customer_type");
        Map<String, String> mapDic = new HashMap<>();
        for (DictDataRespDTO d : dicCustomer_type
        ) {
            mapDic.put(d.getValue(), d.getLabel());
        }

        PageResult<CustomerAnalysisResp> pageResult = getListPage(query);
        String sTemp = "";
        int nPm = query.getStart() + 1;
        for (CustomerAnalysisResp r : pageResult.getList()) {
            r.setPx(nPm);
            nPm++;

            //客户类型转换
            String sType = r.getType();
            if (sType == null) {
                r.setCustomerrole("");
            } else {
                r.setCustomerrole(mapDic.get(sType));
            }

            if (r.getAllsumvolume() == null ||
                    r.getAllsumvolume().compareTo(BigDecimal.ZERO) == 0) {

                r.setAllsumvolume(new BigDecimal(0));

                //计算月均
                r = calMonthAvg(query, r, true);

                //计算总值
                r = calAll(r, true);

                //计算海运数据
                r = cal1(query, r, true);
                //计算空运数据
                r = cal3(query, r, true);
                //计算重货数据
                r = calZH(r, true);
                //计算泡货数据
                r = calPH(r, true);
                //计算控货数据
                r = calKH(r, true);

            } else {
                //计算月均
                r = calMonthAvg(query, r, false);
                //计算总值
                r = calAll(r, false);
                //计算海运数据
                r = cal1(query, r, false);
                //计算空运数据
                r = cal3(query, r, false);
                //重货
                r = calZH(r, false);
                //计算泡货占比
                r = calPH(r, false);
                //计算控货占比
                r = calKH(r, false);
            }
            //计算提货率
            r = calPick(r, false);
        }
        return pageResult;
    }

    public List<CustomerAnalysisResp> GetDataListResult(CustomerAnalysisReq query)
            throws JsonProcessingException {

        query = getReq(query);
        query = getDataWhere(query);
        System.out.println("处理后的查询条件：" + query);

        //把model转 JSON字符串 lanbm
        //String jsonStr = JsonUtils.toJsonString(query);
        //JsonUtils.SaveLog(jsonStr);

        List<DictDataRespDTO> dicCustomer_type =
                dictDataApi.getDictDatas("customer_type");
        Map<String, String> mapDic = new HashMap<>();
        for (DictDataRespDTO d : dicCustomer_type
        ) {
            mapDic.put(d.getValue(), d.getLabel());
        }

        List<CustomerAnalysisResp> list = getList(query);
        String sTemp = "";
        int nPm = query.getStart() + 1;
        for (CustomerAnalysisResp r : list) {
            r.setPx(nPm);
            nPm++;

            //客户类型转换
            String sType = r.getType();
            if (sType == null) {
                r.setCustomerrole("");
            } else {
                r.setCustomerrole(mapDic.get(sType));
            }

            if (r.getAllsumvolume() == null ||
                    r.getAllsumvolume().compareTo(BigDecimal.ZERO) == 0) {

                r.setAllsumvolume(new BigDecimal(0));

                //计算月均
                r = calMonthAvg(query, r, true);

                //计算总值
                r = calAll(r, true);

                //计算海运数据
                r = cal1(query, r, true);
                //计算空运数据
                r = cal3(query, r, true);
                //计算重货数据
                r = calZH(r, true);
                //计算泡货数据
                r = calPH(r, true);
                //计算控货数据
                r = calKH(r, true);

            } else {
                //计算月均
                r = calMonthAvg(query, r, false);
                //计算总值
                r = calAll(r, false);
                //计算海运数据
                r = cal1(query, r, false);
                //计算空运数据
                r = cal3(query, r, false);
                //重货
                r = calZH(r, false);
                //计算泡货占比
                r = calPH(r, false);
                //计算控货占比
                r = calKH(r, false);
            }
            r = calPick(r, false);
        }
        return list;
    }


    private CustomerAnalysisResp calMonthAvg(CustomerAnalysisReq query,
                                             CustomerAnalysisResp r, boolean bZzNull) {
        String sTemp = "";
        if (bZzNull == true) {
            r.setAllsumvolume(new BigDecimal(0));
            r.setMonthAvg("0");
        } else {
            r.setMonthAvg(r.getAllsumvolume().divide(
                    new BigDecimal(query.getNMonth()), 2,
                    BigDecimal.ROUND_HALF_UP).toString());

        }

        //计算同比
        if (r.getAllsumvolumeTb() == null ||
                r.getAllsumvolumeTb().compareTo(BigDecimal.ZERO) == 0) {
            r.setMonthAvgTbShow("同期值为0");
        } else {
            sTemp = getPercentage(
                    (r.getAllsumvolume().subtract(r.getAllsumvolumeTb())),
                    r.getAllsumvolumeTb());
            r.setMonthAvgTbShow(sTemp);
        }
        return r;
    }

    /*
    总值计算
     */
    private CustomerAnalysisResp calAll(CustomerAnalysisResp r, boolean bZzNull) {
        String sTemp = "";
        if (bZzNull == true) {
            r.setAllsumvolume(new BigDecimal(0));
        }

        if (r.getAllsumvolumeTb() == null ||
                r.getAllsumvolumeTb().compareTo(BigDecimal.ZERO) == 0) {
            r.setAllsumvolumeTbShow("同期V值为0");
        } else {
            //(本期-同期)/同期
            sTemp = getPercentage(
                    (r.getAllsumvolume().subtract(r.getAllsumvolumeTb())),
                    r.getAllsumvolumeTb());
            r.setAllsumvolumeTbShow(sTemp);
        }
        return r;
    }


    /*
     计算提货数据
     */
    private CustomerAnalysisResp calPick(CustomerAnalysisResp r, boolean bZzNull) {
        if (r.getChargequantity() == 0) {
            r.setThL("总箱数为0");
        } else {
            r.setThL(getIntPercentage(r.getPickquantity(), r.getChargequantity()));
        }
        return r;
    }

    /*
      处理海运数据
     */
    private CustomerAnalysisResp cal1(CustomerAnalysisReq query,
                                      CustomerAnalysisResp r, boolean bZzNull) {
        String sTemp = "";

        if (bZzNull == true) {
            r.setAllsumvolume(new BigDecimal(0));
        }

        if (r.getSumvolume1() == null ||
                r.getSumvolume1().compareTo(BigDecimal.ZERO) == 0) {
            //海运V值
            r.setSumvolumeV1(new BigDecimal(0));
            //海运体积数
            r.setSumvolume1(new BigDecimal(0));
            r.setSeaMonthAvgZ(new BigDecimal(0));

            if (bZzNull == true) {
                r.setSeaZb("总值为0");
                //海运月均
                r.setSeaMonthAvg("总值为0");
            } else {
                //海运占比
                r.setSeaZb("0");
                //海运月均
                r.setSeaMonthAvg("0");
            }

        } else {
            //设置海运V值
            r.setSumvolumeV1(r.getSumvolume1());

            if (bZzNull == true) {
                r.setSeaZb("总值为0");
            } else {
                r.setSeaZb(getPercentage(r.getSumvolumeV1(), r.getAllsumvolume()));
            }
            //月均V值 是体积
            r.setSeaMonthAvgZ(r.getSumvolume1().divide(new BigDecimal(query.getNMonth()), 2,
                    BigDecimal.ROUND_HALF_UP));

            r.setSeaMonthAvg(r.getSeaMonthAvgZ().toString());
        }

        if (r.getSumvolumeTb1() == null ||
                r.getSumvolumeTb1().compareTo(BigDecimal.ZERO) == 0) {
            r.setSumvolumeTbV1(new BigDecimal(0));

            r.setSumvolumeTbShow1("同期为0");
            r.setSeaMonthAvgTbZ(new BigDecimal(0));

        } else {
            r.setSumvolumeTbV1(new BigDecimal(1).multiply(r.getSumvolumeTb1()));
            sTemp = getPercentage(
                    (r.getSumvolumeV1().subtract(r.getSumvolumeTbV1())),
                    r.getSumvolumeTbV1());
            r.setSumvolumeTbShow1(sTemp);

            r.setSeaMonthAvgTbZ(r.getSumvolumeTb1().divide(new BigDecimal(query.getNMonth()), 2,
                    BigDecimal.ROUND_HALF_UP));
        }


        //计算海运月均同比
        if (r.getSeaMonthAvgTbZ() == null ||
                r.getSeaMonthAvgTbZ().compareTo(BigDecimal.ZERO) == 0) {
            r.setSeaMonthAvgTb("同期为0");
        } else {

            sTemp = getPercentage(
                    (r.getSeaMonthAvgZ().subtract(r.getSeaMonthAvgTbZ())),
                    r.getSeaMonthAvgTbZ());
            r.setSeaMonthAvgTb(sTemp);

        }
        return r;
    }


    /*
     处理空运数据
     */
    private CustomerAnalysisResp cal3(CustomerAnalysisReq query,
                                      CustomerAnalysisResp r, boolean bZzNull) {

        String sTemp = "";
        if (bZzNull == true) {
            r.setAllsumvolume(new BigDecimal(0));
        }

        //当期值
        if (r.getSumweightV3() == null ||
                r.getSumweightV3().compareTo(BigDecimal.ZERO) == 0) {

            r.setSumweightV3(new BigDecimal(0));
            r.setSumweight3(new BigDecimal(0));
            r.setAirMothAvgZ(new BigDecimal(0));

            if (bZzNull == true) {
                r.setAirZb("总值为0");
                r.setAirMothAvg("总值为0");
            } else {
                r.setAirZb("0");
                r.setAirMothAvg("0");
            }
        } else {
            //SQL语句算出的是V值，要把V值转换位KG数
            r.setSumweight3(r.getSumweightV3().multiply(new BigDecimal(100)));
            if (bZzNull == true) {
                r.setAirZb("总值为0");
            } else {
                //空运占比
                r.setAirZb(getPercentage(r.getSumweightV3(), r.getAllsumvolume()));
            }

            //月均是 重量
            r.setAirMothAvgZ(r.getSumweight3().divide(new BigDecimal(query.getNMonth()), 2,
                    BigDecimal.ROUND_HALF_UP));

            r.setAirMothAvg(r.getAirMothAvgZ().toString());
        }

        //同期值
        if (r.getSumweightTbV3() == null ||
                r.getSumweightTbV3().compareTo(BigDecimal.ZERO) == 0) {
            r.setSumweightTb3(new BigDecimal(0));
            r.setSumweightTbV3(new BigDecimal(0));
            r.setSumweightTbShow3("同期值为0");
            r.setAirMothAvgTbZ(new BigDecimal(0));
        } else {
            //SQL语句算出的是V值，要把V值转换位KG数
            r.setSumweightTb3(r.getSumweightTbV3().multiply(new BigDecimal(100)));
            sTemp = getPercentage(
                    (r.getSumweightV3().subtract(r.getSumweightTbV3())),
                    r.getSumweightTbV3());
            r.setSumweightTbShow3(sTemp);

            r.setAirMothAvgTbZ(r.getSumweightTb3().divide(new BigDecimal(query.getNMonth()), 2,
                    BigDecimal.ROUND_HALF_UP));
        }

        //计算空运月均同比
        if (r.getAirMothAvgTbZ() == null ||
                r.getAirMothAvgTbZ().compareTo(BigDecimal.ZERO) == 0) {
            r.setAirMothAvgTb("同期为0");
        } else {
            sTemp = getPercentage(
                    (r.getAirMothAvgZ().subtract(r.getAirMothAvgTbZ())),
                    r.getAirMothAvgTbZ());
            r.setAirMothAvgTb(sTemp);
        }
        return r;
    }

    /*
     计算重货数据
     */
    private CustomerAnalysisResp calZH(CustomerAnalysisResp r, boolean bZzNull) {
        if (bZzNull == true) {
            //总值为0的情况
            r.setAllsumvolume(new BigDecimal(0));
        }

        //重货 占比
        if (r.getWeightSumV() == null ||
                r.getWeightSumV().compareTo(BigDecimal.ZERO) == 0) {
            r.setWeightSumV(new BigDecimal(0));
            if (bZzNull == true) {
                r.setWeithtSumZb("总值为0");
            } else {
                r.setWeithtSumZb("0");
            }

        } else {
            r.setWeithtSumZb(getPercentage(r.getWeightSumV(), r.getAllsumvolume()));
        }
        return r;
    }

    /*
     计算泡货数据
     */
    private CustomerAnalysisResp calPH(CustomerAnalysisResp r, boolean bZzNull) {
        if (bZzNull == true) {
            //泡货 占比
            r.setAllsumvolume(new BigDecimal(0));
        }
        if (r.getPhSumV() == null ||
                r.getPhSumV().compareTo(BigDecimal.ZERO) == 0) {

            r.setPhSumV(new BigDecimal(0));
            if (bZzNull == true) {
                r.setPhSumZb("总值为0");
            } else {
                r.setPhSumZb("0");
            }

        } else {
            r.setPhSumZb(getPercentage(r.getPhSumV(), r.getAllsumvolume()));
        }

        return r;
    }

    /*
     计算控货数据
     */
    private CustomerAnalysisResp calKH(CustomerAnalysisResp r, boolean bZzNull) {
        if (bZzNull == true) {
            //控货 占比
            r.setAllsumvolume(new BigDecimal(0));
        }
        if (r.getKhSumV() == null ||
                r.getKhSumV().compareTo(BigDecimal.ZERO) == 0) {

            r.setKhSumV(new BigDecimal(0));
            if (bZzNull == true) {
                r.setKhSumZb("总值为0");
            } else {
                r.setKhSumZb("0");
            }
        } else {
            r.setKhSumZb(getPercentage(r.getKhSumV(),
                    r.getAllsumvolume()));
        }
        return r;
    }


    /*
     lanbm 2024-05-15 修改提货率计算保留小数点的BUG
     */
    private String getIntPercentage(int number, int divisor) {
        // 计算百分比
        double percentage = number / (double) divisor;
        // 格式化输出百分比
        String formatted = String.format("%.2f%%", percentage*100);
        return formatted;
    }

    private int monthDiff(String dateString1, String dateString2) {
        // 将字符串转换为 LocalDate 对象
        LocalDate date1 = LocalDate.parse(dateString1);
        LocalDate date2 = LocalDate.parse(dateString2);

        // 获软时间1的年份和月份
        int year1 = date1.getYear();
        int month1 = date1.getMonthValue();
        // 获取时间2的年份和月份
        int year2 = date2.getYear();
        int month2 = date2.getMonthValue();

        // 计算时间1的总月数
        int totalMonths1 = year1 * 12 + month1;
        // 计算时间2的总月数
        int totalMonths2 = year2 * 12 + month2;
        // 计算总月数差
        int monthsDiff = totalMonths2 - totalMonths1;

        return monthsDiff;
    }


    private List<CustomerAnalysisResp> GetPageResult2(
            List<CustomerAnalysisResp> list, int pageNum, int pageSize) {
        /*
        // 正序
        Collections.sort(list, (a, b) -> {
            return a.排序字段 - b.排序字段;
        });
        // 倒序
        Collections.sort(list, (a, b) -> {
            return b.排序字段 - a.排序字段;
        });*/
        // 总数
        int total = list.size();

        // 分页数据
        List<CustomerAnalysisResp> subList =
                list.stream().skip((pageNum - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
        return subList;

    }



    /*
     lanbm 2024-04-26 add
     */
    public PageResult<CustomerAnalysisResp> getListPage_New(CustomerAnalysisReq req)
            throws JsonProcessingException {
        List<CustomerAnalysisResp> list = customerAnalysisMapper.getListPage_New(req);
        long  total=list.size();
        list = GetPageResult(list, req.getPageNo(), req.getPageSize());
        return new PageResult<>(list, total, req.getPageSize(),
                req.getPageNo(),
                (total + req.getPageSize() - 1) / req.getPageSize());

    }

}
